from typing import List

from PyQt5.QtCore import QTranslator
from PyQt5.QtWidgets import QApplication, QMessageBox

from astra_installer.settings.component import has_component_with_ident
from astra_installer.settings.config import ConfigLoader
from astra_installer.settings.oem_manager import SettingsManager, OEMSettingsManager
from astra_installer_qt.mainwindow import MainWindowCommon
from astra_installer_qt_oem.utils import Utils

translate = QApplication.translate


class MainWindowOEM(MainWindowCommon):
    def __init__(self, locale: str):
        super(MainWindowOEM, self).__init__(locale)
        self.setup()

    def load_settings(self):
        config = ConfigLoader(oem=True).load()
        OEMSettingsManager.create_settings(config)
        if self.locale in SettingsManager.installer_locale.locales:
            SettingsManager.installer_locale.selected = self.locale

    def setup(self):
        # В режиме OEM разметка диска отключена
        self.labelStorageSettings.hide()
        self.pushButtonStorageSettings.hide()
        self.labelStorageInfo.hide()
        # TODO: Скрытие ломает выравнивание. Spacer не помогает
        # self.labelMode.hide()
        # self.comboBoxMode.hide()
        # object has no attribute 'horizontalSpacerMode'
        # self.horizontalSpacerMode.hide()
        self.labelMode.setEnabled(False)
        self.comboBoxMode.setEnabled(False)
        if not SettingsManager.systeminfo.is_kiosk_session:
            self.labelUsername.hide()
            self.lineEditUsername.hide()
            self.pages_manager.open_page2()
            self.pushButtonBack.setEnabled(False)
        self.install_messagebox_title = translate('MainWindowCommon', 'Setup confirmation')
        self.installer_language.change_installer_locale()

    @property
    def install_messagebox_question(self) -> str:
        return translate('MainWindowCommon', 'Are you really sure you want to apply these settings?')

    @property
    def os_components_text(self) -> str:
        return translate('MainWindowCommon', 'Select additional OS components')

    @property
    def help_path(self) -> str:
        return Utils.file_path('resources', 'help')

    @property
    def window_title(self) -> str:
        return translate('title', 'Astra Linux initial system setup')

    @property
    def password_label_text(self) -> str:
        if SettingsManager.systeminfo.is_kiosk_session:
            return translate('text', 'Administrator password:')
        else:
            return translate('text', '{username} password:').format(username=SettingsManager.user.username)

    @property
    def reboot_checkbox_text(self) -> str:
        return translate('text', 'Reboot computer after setup is complete')

    @property
    def sucessfully_installed_text(self) -> str:
        return translate('text', 'You have successfully setup')

    @property
    def install_button_text(self) -> str:
        return translate('text', 'Setup')

    def check_settings(self) -> None:
        tasks = self.os_components_dialog.components_manager.model.checked_components()
        features = self.security_components_dialog.components_manager.model.checked_components()
        warnings = []  # type: List[str]
        if has_component_with_ident('astra-digsig-control', features):
            warnings.append(translate('MainWindowCommon',
                                      'You have selected ELF signature check. '
                                      'After a reboot, all unsigned binaries cannot be executed.'))
        if has_component_with_ident('Fly-virtualization', tasks) \
                and has_component_with_ident('astra-ufw-control', features):
            warnings.append(translate('MainWindowCommon',
                                      'You have selected the OS component "Virtualization tools" and '
                                      'additional OS setting "Enable ufw firewall". '
                                      'These components may be incompatible.'))
        if warnings:
            title = translate('MainWindowCommon', self.install_messagebox_title)
            question = translate('MainWindowCommon',
                                 'Warnings:\n{warnings}\nAre you really sure you want to apply these settings?')
            if QMessageBox.question(self, title, question.format(warnings='\n'.join(warnings))) == QMessageBox.Yes:
                self.setup_backend()
                self.start_installation()
        else:
            self.setup_backend()
            self.ask_about_install()
